/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lkwueberholverbotsichtwochentag;

import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataAndATGUsageInformation;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmen.lib.DefinitionenGlobal;
import de.bsvrz.ste.stealgsba.algorithmen.lib.EinzelsituationVorstufeEmpfang;
import de.bsvrz.ste.stealgsba.algorithmen.lib.INeueDatenListener;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SenderOhneStatusRueckmeldung;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SimpleSenderMitWartenAufParametrierung;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseMq;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseMqEmpfang;
import de.bsvrz.ste.stealgsba.algorithmen.lib.StaukriteriumErgebnis;
import de.bsvrz.ste.stealgsba.algorithmen.lib.TestTools;
import de.bsvrz.ste.stealgsba.algorithmen.lkwueberholverbotsichtwochentag.LkwUeberholverbotSichtWochentagAlg;
import de.bsvrz.ste.stealgsba.algorithmen.lkwueberholverbotsichtwochentag.ParameterLkwUeberholverbotSichtWochentag;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.AnzeigenDatensatz;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.EinzelMassnahmeAlternative;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Massnahmenanforderung;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.exceptions.SenderException;
import de.bsvrz.sys.funclib.vewdynobj.VerwaltungDynObj;

/**
 * Klasse fr die Durchfhrung der Prfflle 13.16 und 13.17 wie in der Prfspezifikation beschrieben.
 *
 * @version $Revision: 1.2 $ / $Date: 2015/09/11 07:27:30 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class LkwUeberholverbotSichtWochentagTest implements INeueDatenListener
{
  /**
   * Datenverteilerverbindung.
   */
  private static ClientDavInterface m_connection = null;
  /**
   * Aktuelles Datenmodell.
   */
  private static DataModel m_dataModel;
  
  /**
   * Typ der Ursacheneinheit.
   */
  private static String m_typUe;
  /**
   * Pid der Ursacheneinheit.
   */
  private static String m_pidUe;
  /**
   * Name der Ursacheneinheit.
   */
  private static String m_nameUe;
  /**
   * Pid der Streckenbeeinflussungsanlage.
   */
  private static String m_anlage;
  /**
   * Synchronisierungsobjekt.
   */
  private static Object m_objekt;
  /**
   * Synchronisierungsobjekt.
   */
  private static Object m_objekt2;
  /**
   * Pid der Anzeige.
   */
  private static String m_pidAnzeige;
  /**
   * Pid des Messquerschnitt.
   */
  private static String m_pidMessQuerschnitt;	
  /**
   * Pid des Umfelddatenmessstelle.
   */
  private static String m_pidUmfelddatenSensor;	
  /**
   * Datensatz Einzelsituationvorstufe.
   */
  private static ResultData m_DatensatzEinzelsituationVorstufe = null;
	/**
	 * Analysedaten.
	 */
	private static SituationAnalyseMq m_analyseMq = null;	
  /**
   * Zugriff auf die Instanz des Algorithmus.
   */
  private static LkwUeberholverbotSichtWochentagAlg m_alg;
  /**
   * Zugriff auf die Instanz der Verwaltung dynamischer Objekte.
   */
  private static VerwaltungDynObj m_vdo;  
	/**
	 * Liste mit Fahrstreifenobjekten.
	 */
	private static List<SystemObject> m_fahrstreifen = new ArrayList<SystemObject>();
	/**
	 * Liste mit Messquerschnitten.
	 */
	private static List<SystemObject> m_messquerschnitte = new ArrayList<SystemObject>();	
	/**
	 * Liste mit Umfelddatenmessstellen.
	 */
	private static List<SystemObject> m_umfeldDatenSensoren = new ArrayList<SystemObject>();	
  /**
   * Flag Daten erhalten.
   */
  private static boolean m_datenErhalten = false;
	/**
	 * Flag Analysedaten erhalten.
	 */
	private static boolean m_analyseMqErhalten = false;
	/**
	 * Kennung Prffall 1.
	 */
	private final static int m_prueffall1 = 1;
	/**
	 * Kennung Prffall 2.
	 */
	private final static int m_prueffall2 = 2;
	/**
	 * Pid EreignisTyp.
	 */
	private static String m_pidEreignisTyp;
	/**
	 * Objekt EreignisTyp.
	 */
	private static SystemObject m_ereignisTyp;
	/**
	 * Pid Systemkalendereintrag.
	 */
	private static String m_pidSysKalEintrag;
	/**
	 * Pid Ereignis.
	 */
	private static String m_pidEreignis;
	/**
	 * Objekt Ereignis.
	 */
	private static SystemObject m_ereignis;
	/**
	 * Pid Lalender.
	 */
	private static String m_pidKalender;
	
  /**
   * Initialisiert die Testumgebung.
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  	// Globale Daten setzen
    m_pidAnzeige = "bw.uz.B27.streckenBeeinflussungsAnlage.testAnzeigeQuerschnitt1.anzeige1";
    m_pidMessQuerschnitt = "mq.MQ.B27.4202.HFB.N";
    m_pidUmfelddatenSensor = "UFDS.HS.B27.4103.S.SW";
    m_anlage = DefinitionenGlobal.getAnlage();
    m_typUe = "typ.sswSbaUeLkwberholverbotSichtWochentag";
    m_pidUe = "bw.uz.B27.UeLkwberholverbotSichtWochentag.JUnitTest1";
    m_nameUe = "UrsachenEinheitLkwberholverbotSichtWochentag.JUnitTest1";

    // Debug-Level setzen    
    Debug.setLoggerLevel("DEFAULT", Debug.OFF);
    Debug.setHandlerLevel("StdErr", Debug.OFF);
    
    // Datenverteilerverbindung aufbauen    
    ClientDavParameters parameters = new ClientDavParameters();
    parameters.setDavCommunicationAddress(DefinitionenGlobal.getDavCommunicationAddress());
    parameters.setDavCommunicationSubAddress(DefinitionenGlobal.getDavCommunicationSubAddress());
    parameters.setUserName(DefinitionenGlobal.getUserName());
    parameters.setUserPassword(DefinitionenGlobal.getUserPassword());

    m_connection = new ClientDavConnection(parameters);
    m_connection.connect();
    m_connection.login();
    m_dataModel = m_connection.getDataModel();
    
    // Synchronisierungsobjekte erstellen
    m_objekt = new Object();
    m_objekt2 = new Object();
    
    // Aufrufparameter definieren
    String[] argumente = 
    {
     	DefinitionenGlobal.getArgumentDav(),
     	DefinitionenGlobal.getArgumentBenutzer(),
     	DefinitionenGlobal.getArgumentDebugLevelStdErrText(),
     	DefinitionenGlobal.getArgumentAuthentifizierung(),
  		"-Anlage=" + DefinitionenGlobal.getAnlage(),
  		"-AspektVersand=asp.sswSbaLkwberholverbotSichtWochentag",
  		DefinitionenGlobal.getMaxZyklusDauer(),
  		DefinitionenGlobal.getZyklusLaenge(),
  		DefinitionenGlobal.getDeltaPruefen(),
  		"-Spontan=true"
    };
    
    // Objekte erstellen
		m_messquerschnitte.add(m_dataModel.getObject(m_pidMessQuerschnitt));

		for (int i = 0; i < m_messquerschnitte.size(); i++)
		{
			m_fahrstreifen = bildeListeFahrstreifen(m_messquerschnitte);			
		}

		m_umfeldDatenSensoren.add(m_dataModel.getObject(m_pidUmfelddatenSensor));

		m_ereignisTyp = m_dataModel.getObject(m_pidEreignisTyp);
		m_ereignis = m_dataModel.getObject(m_pidEreignis);
		
		m_pidKalender = m_dataModel.getConfigurationAuthority().getPid();
		
		// Algorithmus instanziieren und starten
    m_alg = new LkwUeberholverbotSichtWochentagAlg();
    
    StandardApplicationRunner.run(m_alg, argumente);
    
    // UE erzeugen und in Menge legen            
    m_vdo = new VerwaltungDynObj();
    m_vdo.setConnection(m_connection);
    
    try
		{
    	m_vdo.reduziereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);
			m_vdo.loescheObjekt(m_pidUe);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

    try
		{
			m_vdo.erzeugeObjekt(m_typUe, m_pidUe, m_nameUe, null);
			m_vdo.erweitereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);
		}
		catch (Exception e1)
		{
			e1.printStackTrace();
			
      fail("Ursacheneinheit konnte nicht erstellt werden");
		}
		// Systemkalendereintrag erstellen
    
		if (m_dataModel.getObject(m_pidSysKalEintrag) == null)
		{
	    m_vdo.erzeugeObjekt("typ.systemKalenderEintrag", m_pidSysKalEintrag, "Test", null);
			m_vdo.erweitereMenge(m_pidSysKalEintrag, "SystemKalenderEintrge", m_pidKalender);
	
			AttributeGroup attributgruppe = m_dataModel.getAttributeGroup("atg.systemKalenderEintrag");
	    Aspect aspekt = m_dataModel.getAspect("asp.parameterVorgabe");
	    
	    Data data = m_connection.createData(attributgruppe);
	    data.getTextValue("Definition").setText("Tag");
	
	    DataDescription dd = new DataDescription(attributgruppe, aspekt);
	
			SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, m_dataModel.getObject(m_pidSysKalEintrag), dd, data, 60000);
		}
		
		// Ereignistyp erstellen
    
		if (m_ereignisTyp == null)
		{
			AttributeGroup _attributgruppe = m_dataModel.getAttributeGroup("atg.ereignisTypEigenschaften");
	    Aspect _aspekt = m_dataModel.getAspect("asp.eigenschaften");

	    Collection<DataAndATGUsageInformation> datas = new Vector<DataAndATGUsageInformation>();
	    Data data2 = m_connection.createData(_attributgruppe);
	    data2.getArray("ZustzlicheAttribute").setLength(0);
	    
	    AttributeGroupUsage agu = m_dataModel.getAttributeGroupUsage(_attributgruppe.getAttributeGroupUsage(_aspekt).getId());
	    
	    DataAndATGUsageInformation dataAtg = new DataAndATGUsageInformation(agu, data2);
	    datas.add(dataAtg);

	    m_vdo.erzeugeObjekt("typ.ereignisTyp", m_pidEreignisTyp, "Test", datas);
			m_vdo.erweitereMenge(m_pidEreignisTyp, "EreignisTypen", m_pidKalender);		
		}

		m_ereignisTyp = TestTools.getDynObjAusMenge(m_connection, m_pidKalender, "EreignisTypen", m_pidEreignisTyp);
		
		// Ereignis erstellen

		if (m_ereignis == null)
		{
			AttributeGroup _attributgruppe = m_dataModel.getAttributeGroup("atg.ereignisEigenschaften");
	    Aspect _aspekt = m_dataModel.getAspect("asp.eigenschaften");
	
	    Collection<DataAndATGUsageInformation> datas = new Vector<DataAndATGUsageInformation>();
	    Data data2 = m_connection.createData(_attributgruppe);
	    data2.getReferenceValue("EreignisTypReferenz").setText(m_pidEreignisTyp);
	    data2.getTextValue("Ereignisbeschreibung").setText("");
	    data2.getArray("ZustzlicheAttribute").setLength(0);
	    
	    AttributeGroupUsage agu = m_dataModel.getAttributeGroupUsage(_attributgruppe.getAttributeGroupUsage(_aspekt).getId());
	    
	    DataAndATGUsageInformation dataAtg = new DataAndATGUsageInformation(agu, data2);
	    datas.add(dataAtg);
	    
			m_vdo.erzeugeObjekt("typ.ereignis", m_pidEreignis, "Test", datas);
			m_vdo.erweitereMenge(m_pidEreignis, "Ereignisse", m_pidKalender);
		}
  	
  	m_ereignis = TestTools.getDynObjAusMenge(m_connection, m_pidKalender, "Ereignisse", m_pidEreignis);
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  	m_vdo.reduziereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);
  	m_vdo.loescheObjekt(m_pidUe);

  	m_vdo.reduziereMenge(m_pidEreignis, "Ereignisse", m_pidKalender);
		m_vdo.loescheObjekt(m_pidEreignis);

  	m_vdo.reduziereMenge(m_pidEreignisTyp, "EreignisTypen", m_pidKalender);
		m_vdo.loescheObjekt(m_pidEreignisTyp);

		m_vdo.reduziereMenge(m_pidSysKalEintrag, "SystemKalenderEintrge", m_pidKalender);
		m_vdo.loescheObjekt(m_pidSysKalEintrag);
  	
  	m_connection.disconnect(false, "");    
  }


  @Before
  public void setUp() throws Exception
  {
  }

  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * In diesem Testfall werden die in der Prfspezifikation definierten Testflle
   * fr den Prffall 13.16 durchgefhrt.
   */
  @Test
  public final void prueffall_13_16 ()
  {
  	prueffall_13_16_Vorbereitung();
  	prueffall_13_16_Parametrierung(m_prueffall1);
  	prueffall_13_16_Test_1(m_prueffall1);
  	prueffall_13_16_Test_2(m_prueffall1);
  	prueffall_13_16_Test_3(m_prueffall1);
  	prueffall_13_16_Test_4(m_prueffall1);
  	prueffall_13_16_Test_5(m_prueffall1);
  }

  /**
   * In diesem Testfall werden die in der Prfspezifikation definierten Testflle
   * fr den Prffall 13.17 durchgefhrt.
   */
  @Test
  public final void prueffall_13_17 ()
  {
  	prueffall_13_16_Vorbereitung();
  	prueffall_13_16_Parametrierung(m_prueffall2);
  	prueffall_13_16_Test_1(m_prueffall2);
  	prueffall_13_16_Test_2(m_prueffall2);
  	prueffall_13_16_Test_3(m_prueffall2);
  	prueffall_13_16_Test_4(m_prueffall2);
  	prueffall_13_16_Test_5(m_prueffall2);
  }

  /**
   * In diesem Testfall wird berprft, ob die fr den JUnitTest notwendige Ursacheneinheit
   * korrekt erstellt wurde und vom Steuerungsalgorithmus bearbeitet wird. berprft wird
   * hierzu der Empfang der Statusinformation "EinzelsituationVorstufe", die nach dem Berechnen
   * versendet wird.
   */
  private void prueffall_13_16_Vorbereitung ()
  {
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);
  	
    EinzelsituationVorstufeEmpfang esve = new EinzelsituationVorstufeEmpfang(m_connection, ue);
    esve.addListener(this);
    
    m_alg.starteBerechnung();
    
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(65000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
    		
		if (!m_datenErhalten)
			fail ("Vorbereitung EinzelsituationVorstufe nicht empfangen, Ursacheneinheit nicht korrekt erstellt");

    SituationAnalyseMqEmpfang samqe = new SituationAnalyseMqEmpfang(m_connection, ue);
    samqe.addListener(this);

  }
  
  /**
   * In diesem Tesfall werden folgende Attribitgruppen der Ursacheneinheit parametriert.<p>
   * - atg.sswSbaUeLkwberholverbotSichtWochentag<br>
   * - atg.sswSbaEinzelMassnahme<br>
   * - atg.sswSbaUrsachenEinheit<br>
   * - atg.sswSbaUeMqZuordnung<p>
   * - atg.sswSbaUeUfdsZuordnung<p>
   */
  private void prueffall_13_16_Parametrierung(int prueffall)
  {
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);

  	SystemObject anzeige = m_dataModel.getObject(m_pidAnzeige);
  	
  	Data atgParam = erzeugeAtgParameterLkwUeberholverbotSichtWochentag(prueffall);
  	Data atgEm = erzeugeAtgEinzelMassnahme(erzeugeMassnahmenAnforderung(anzeige));
  	Data atgUe = erzeugeAtgUrsachenEinheit();
  	Data atgMq = erzeugeAtgMqZuordnung();
  	Data atgUfds = erzeugeAtgUfdsZuordnung();
  	
		try
		{
			parametriereDynObjUrsachenEinheit(ue, atgParam, atgEm, atgUe, atgMq, atgUfds);
		}
		catch (Exception e)
		{
			fail ("Ursacheneinheit konnte nicht parametriert werden");
		}

//		m_connection.sleep(500000);
  }
  
  /**
   * In dieser Methode wird der Testfall 13.16 (1) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_16_Test_1(int prueffall)
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;
		
		int qB = 2800;
		int qLkw = 540;
		int stufe = 1;

		boolean zeitlichGueltig = true;
		boolean verkehrlichGltig = false;
				
		try
		{
			Data prognoseDatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, -1, -1, qB, -1, qLkw);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, prognoseDatenMq);				
			}

			Data kurzZeitDatenMq = erzeugeAtgVerkehrsDatenKurzZeitMq(-1, -1, qB, -1);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitMq"), m_dataModel.getAspect("asp.analyse"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, kurzZeitDatenMq);				
			}
			
			Data sichtStufe = erzeugeAtgUfdsSichtStufe(stufe);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ufdsStufeSichtWeite"), m_dataModel.getAspect("asp.klassifizierung"));

			for (int i = 0; i < m_umfeldDatenSensoren.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_umfeldDatenSensoren.get(i), dd, sichtStufe);				
			}

			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ereignis"), m_dataModel.getAspect("asp.zustand"));
			SenderOhneStatusRueckmeldung.source(m_connection, m_ereignis, dd, erzeugeAtgEreignis(zeitlichGueltig, verkehrlichGltig));							
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
    
		if (m_analyseMqErhalten)
		{
			if (m_analyseMq != null)
			{
				if (m_analyseMq.isEinschaltkriterium())
				{
					fail("Test 1 -> Einschaltkriterium nicht korrekt");
				}

				if (!m_analyseMq.isAusschaltkriterium())
				{
					fail("Test 1 -> Ausschaltkriterium nicht korrekt");
				}
			}
		}
		
		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 1 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }
  

  /**
   * In dieser Methode wird der Testfall 13.16 (2) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_16_Test_2(int prueffall)
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int qB = 3000;
		int qLkw = 560;
		int stufe = 3;

		boolean zeitlichGueltig = true;
		boolean verkehrlichGltig = false;
				
		try
		{
			Data prognoseDatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, -1, -1, qB, -1, qLkw);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, prognoseDatenMq);				
			}

			Data kurzZeitDatenMq = erzeugeAtgVerkehrsDatenKurzZeitMq(-1, -1, qB, -1);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitMq"), m_dataModel.getAspect("asp.analyse"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, kurzZeitDatenMq);				
			}
			
			Data sichtStufe = erzeugeAtgUfdsSichtStufe(stufe);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ufdsStufeSichtWeite"), m_dataModel.getAspect("asp.klassifizierung"));

			for (int i = 0; i < m_umfeldDatenSensoren.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_umfeldDatenSensoren.get(i), dd, sichtStufe);				
			}

			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ereignis"), m_dataModel.getAspect("asp.zustand"));
			SenderOhneStatusRueckmeldung.source(m_connection, m_ereignis, dd, erzeugeAtgEreignis(zeitlichGueltig, verkehrlichGltig));							
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_analyseMqErhalten)
		{
			if (m_analyseMq != null)
			{
				if (prueffall == m_prueffall1)
				{
					if (m_analyseMq.isEinschaltkriterium())
					{
						fail("Test 2 -> Einschaltkriterium nicht korrekt");
					}
	
					if (m_analyseMq.isAusschaltkriterium())
					{
						fail("Test 2 -> Ausschaltkriterium nicht korrekt");
					}
				}
				else
				{
					if (m_analyseMq.isEinschaltkriterium())
					{
						fail("Test 2 -> Einschaltkriterium nicht korrekt");
					}
	
					if (!m_analyseMq.isAusschaltkriterium())
					{
						fail("Test 2 -> Ausschaltkriterium nicht korrekt");
					}					
				}
			}
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 2 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }

  /**
   * In dieser Methode wird der Testfall 13.16 (3) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_16_Test_3(int prueffall)
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;


		int qB = 3300;
		int qLkw = 660;
		int stufe = 3;

		boolean zeitlichGueltig = true;
		boolean verkehrlichGltig = false;
				
		try
		{
			Data prognoseDatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, -1, -1, qB, -1, qLkw);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, prognoseDatenMq);				
			}

			Data kurzZeitDatenMq = erzeugeAtgVerkehrsDatenKurzZeitMq(-1, -1, qB, -1);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitMq"), m_dataModel.getAspect("asp.analyse"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, kurzZeitDatenMq);				
			}
			
			Data sichtStufe = erzeugeAtgUfdsSichtStufe(stufe);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ufdsStufeSichtWeite"), m_dataModel.getAspect("asp.klassifizierung"));

			for (int i = 0; i < m_umfeldDatenSensoren.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_umfeldDatenSensoren.get(i), dd, sichtStufe);				
			}

			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ereignis"), m_dataModel.getAspect("asp.zustand"));
			SenderOhneStatusRueckmeldung.source(m_connection, m_ereignis, dd, erzeugeAtgEreignis(zeitlichGueltig, verkehrlichGltig));							
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_analyseMqErhalten)
		{
			if (m_analyseMq != null)
			{
				if (prueffall == m_prueffall1)
				{
					if (!m_analyseMq.isEinschaltkriterium())
					{
						fail("Test 3 -> Einschaltkriterium nicht korrekt");
					}
	
					if (m_analyseMq.isAusschaltkriterium())
					{
						fail("Test 3 -> Ausschaltkriterium nicht korrekt");
					}
				}
				else
				{
					if (m_analyseMq.isEinschaltkriterium())
					{
						fail("Test 3 -> Einschaltkriterium nicht korrekt");
					}
	
					if (!m_analyseMq.isAusschaltkriterium())
					{
						fail("Test 3 -> Ausschaltkriterium nicht korrekt");
					}					
				}
			}
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);

			if (prueffall == m_prueffall1)
			{
				if (!status)
				{
					fail ("Test 3 -> Status der Einzelsituationvorstufe nicht korrekt");				
				}
			}
			else
			{
				if (status)
				{
					fail ("Test 3 -> Status der Einzelsituationvorstufe nicht korrekt");				
				}				
			}
		}
  }
	
  /**
   * In dieser Methode wird der Testfall 13.16 (4) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_16_Test_4(int prueffall)
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int qB = 2900;
		int qLkw = 560;
		int stufe = 3;

		boolean zeitlichGueltig = true;
		boolean verkehrlichGltig = false;
				
		try
		{
			Data prognoseDatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, -1, -1, qB, -1, qLkw);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, prognoseDatenMq);				
			}

			Data kurzZeitDatenMq = erzeugeAtgVerkehrsDatenKurzZeitMq(-1, -1, qB, -1);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitMq"), m_dataModel.getAspect("asp.analyse"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, kurzZeitDatenMq);				
			}
			
			Data sichtStufe = erzeugeAtgUfdsSichtStufe(stufe);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ufdsStufeSichtWeite"), m_dataModel.getAspect("asp.klassifizierung"));

			for (int i = 0; i < m_umfeldDatenSensoren.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_umfeldDatenSensoren.get(i), dd, sichtStufe);				
			}

			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ereignis"), m_dataModel.getAspect("asp.zustand"));
			SenderOhneStatusRueckmeldung.source(m_connection, m_ereignis, dd, erzeugeAtgEreignis(zeitlichGueltig, verkehrlichGltig));							
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_analyseMqErhalten)
		{
			if (m_analyseMq != null)
			{
				if (prueffall == m_prueffall1)
				{
					if (m_analyseMq.isEinschaltkriterium())
					{
						fail("Test 4 -> Einschaltkriterium nicht korrekt");
					}
	
					if (m_analyseMq.isAusschaltkriterium())
					{
						fail("Test 4 -> Ausschaltkriterium nicht korrekt");
					}
				}
				else
				{
					if (m_analyseMq.isEinschaltkriterium())
					{
						fail("Test 4 -> Einschaltkriterium nicht korrekt");
					}
	
					if (!m_analyseMq.isAusschaltkriterium())
					{
						fail("Test 4 -> Ausschaltkriterium nicht korrekt");
					}					
				}
			}
		}

		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			
			if (prueffall == m_prueffall1)
			{
				if (!status)
				{
					fail ("Test 4 -> Status der Einzelsituationvorstufe nicht korrekt");				
				}
			}
			else
			{
				if (status)
				{
					fail ("Test 4 -> Status der Einzelsituationvorstufe nicht korrekt");				
				}				
			}
		}
  }

  /**
   * In dieser Methode wird der Testfall 13.16 (5) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_16_Test_5(int prueffall)
  {
		m_datenErhalten = false;
		m_analyseMqErhalten = false;

		int qB = 2500;
		int qLkw = 10;
		int stufe = 2;

		boolean zeitlichGueltig = true;
		boolean verkehrlichGltig = false;
				
		try
		{
			Data prognoseDatenMq = erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(-1, -1, -1, qB, -1, qLkw);
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"), m_dataModel.getAspect("asp.prognoseNormal"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, prognoseDatenMq);				
			}

			Data kurzZeitDatenMq = erzeugeAtgVerkehrsDatenKurzZeitMq(-1, -1, qB, -1);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitMq"), m_dataModel.getAspect("asp.analyse"));

			for (int i = 0; i < m_messquerschnitte.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_messquerschnitte.get(i), dd, kurzZeitDatenMq);				
			}
			
			Data sichtStufe = erzeugeAtgUfdsSichtStufe(stufe);
			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ufdsStufeSichtWeite"), m_dataModel.getAspect("asp.klassifizierung"));

			for (int i = 0; i < m_umfeldDatenSensoren.size(); i++)
			{
				SenderOhneStatusRueckmeldung.source(m_connection, m_umfeldDatenSensoren.get(i), dd, sichtStufe);				
			}

			dd = new DataDescription(m_dataModel.getAttributeGroup("atg.ereignis"), m_dataModel.getAspect("asp.zustand"));
			SenderOhneStatusRueckmeldung.source(m_connection, m_ereignis, dd, erzeugeAtgEreignis(zeitlichGueltig, verkehrlichGltig));							
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

		if (m_analyseMqErhalten)
		{
			if (m_analyseMq != null)
			{
				if (m_analyseMq.isEinschaltkriterium())
				{
					fail("Test 5 -> Einschaltkriterium nicht korrekt");
				}

				if (!m_analyseMq.isAusschaltkriterium())
				{
					fail("Test 5 -> Ausschaltkriterium nicht korrekt");
				}
			}
		}
    
		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 5 -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}
  }

  /**
   * Erzeugt einen Datensatz der Attributgruppe 'atg.ereignis'.
   * 
   * @param zeitlichGueltig Flag zeitlich gltig.
   * @param verkehrlichGltig Flag verkehrlich gltig.
   * @return erzeugter Datensatz.
   */
  private Data erzeugeAtgEreignis(boolean zeitlichGueltig, boolean verkehrlichGltig)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.ereignis"));

		dataNeu.getScaledValue("zeitlichGltig").setText((zeitlichGueltig) ? "Ja" : "Nein");
		dataNeu.getScaledValue("verkehrlichGltig").setText((verkehrlichGltig) ? "Ja" : "Nein");
		dataNeu.getScaledValue("Attributnderung").setText("Nein");
		dataNeu.getTimeValue("Zeitpunkt").setMillis(new Date().getTime());
		
		return dataNeu;
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.INeueDatenListener#neueDaten(de.bsvrz.dav.daf.main.ResultData)
	 */
	@Override
	public void neueDaten(ResultData rdata)
	{
//		System.out.println("LkwUeberholverbotBelastungWochentagTest.neueDaten() " + rdata.getObject().getPid());
		
		m_datenErhalten = true;
		m_DatensatzEinzelsituationVorstufe = rdata;
		
		synchronized (m_objekt)
		{
			m_objekt.notify();					
		}
	}
  
  /**
   * Erzeugt eine Liste mit Fahrstreifen fr die bergebene Liste der Messquerschnitte.
   * 
   * @param mqs Liste mit Messquerschnitten.
   * @return Liste mit Fahrstreifenobjekte.
   */
  private static List<SystemObject> bildeListeFahrstreifen(List<SystemObject> mqs)
	{
		if (mqs != null)
		{
			List<SystemObject> listeFs = new ArrayList<SystemObject>();
			
			for (int i = 0; i < mqs.size(); i++)
			{
				listeFs.addAll(bestimmeFahrstreifenMq(mqs.get(i)));
			}

			return listeFs;
		}
		return null;
	}
  
  /**
   * Bestimmt die Fahrstreifen des bergebenen Messquerschnitts.
   * 
   * @param mq Messquerschnitt.
   * @return Liste der Fahrstreifenobjekte.
   */
  private static List<SystemObject> bestimmeFahrstreifenMq(SystemObject mq)
	{
		if (mq != null)
		{			
				ConfigurationObject coObj = (ConfigurationObject)mq;
				return coObj.getNonMutableSet("FahrStreifen").getElements();
		}
		return null;
	}

	/**
	 * Bestimmt den aktuellen Zustand der Einzelsituationvorstufe des empfangenen Datensatzes.
	 * 
	 * @return true wenn Einzelsituationvorstufe wahr, ansonsten false.
	 */
	private boolean bestimmeStatusEinzelsituationVorstufe()
	{
		if (m_DatensatzEinzelsituationVorstufe != null)
		{
			if (m_DatensatzEinzelsituationVorstufe.hasData())
			{				
				Data data = m_DatensatzEinzelsituationVorstufe.getData();
				
				return data.getScaledValue("Status").getValueText().equals("Ja");
			}
		}
		
		return false;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaEinzelMassnahme</code>
	 * und gibt ihn zurck.
	 * 
	 * @param massnahmenanforderung Daten der Massnahmeanforderung.
	 * @return Datensatz <code>atg.sswSbaEinzelMassnahme</code>.
	 */
	private Data erzeugeAtgEinzelMassnahme(Massnahmenanforderung massnahmenanforderung)
	{
		if (massnahmenanforderung != null)
		{				
			Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaEinzelMassnahme"));

			TestTools.fuelleAtgEinzelMassnahme(massnahmenanforderung, dataNeu);
			
			return dataNeu;
		}
		
		return null;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUrsachenEinheit</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUrsachenEinheit</code>.
	 */
	private Data erzeugeAtgUrsachenEinheit()
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaUrsachenEinheit"));
		
		TestTools.fuelleAtgUrsachenEinheit(dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeMqZuordnung</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUeMqZuordnungt</code>.
	 */
  private Data erzeugeAtgMqZuordnung()
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaUeMqZuordnung"));

		List<SystemObject> mqs = new ArrayList<SystemObject>();
		mqs.add(m_dataModel.getObject(m_pidMessQuerschnitt));
		
		TestTools.fuelleAtgMqZuordnung(mqs, dataNeu);
		
		System.out.println("MQ-Zuordnmung " + dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeUfdsZuordnung</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUeUfdsZuordnungt</code>.
	 */
  private Data erzeugeAtgUfdsZuordnung()
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaUeUfdsZuordnung"));

		List<SystemObject> mqs = new ArrayList<SystemObject>();
		mqs.add(m_dataModel.getObject(m_pidUmfelddatenSensor));
		
		TestTools.fuelleAtgUfdsZuordnung(mqs, dataNeu);
		
		System.out.println("UFDMS-Zuordnmung " + dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.verkehrsDatenKurzZeitMq</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.verkehrsDatenKurzZeitMq</code>.
	 */
  private Data erzeugeAtgVerkehrsDatenKurzZeitMq(int qkfz, int vkfz, int qb, int aLkw)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitMq"));
		
		TestTools.fuelleAtgVerkehrsDatenKurzZeitMq(qkfz, vkfz, qb, aLkw, dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.verkehrsDatenKurzZeitTrendExtraPolationMq</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.verkehrsDatenKurzZeitTrendExtraPolationMq</code>.
	 */
  private Data erzeugeAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(int qkfz, int vkfz, int vpkw, int qb, int kkfz, int qLkw)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"));
		
		TestTools.fuelleAtgVerkehrsDatenKurzZeitTrendExtraPolationMq(qkfz, vkfz, vpkw, qb, kkfz, qLkw, dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.ufdsStufeSichtWeiteq</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.ufdsStufeSichtWeite</code>.
	 */
  private Data erzeugeAtgUfdsSichtStufe(int ns)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.ufdsStufeSichtWeite"));
	
		dataNeu.getUnscaledValue("Stufe").set(ns);
				
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeLkwberholverbotSicht</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUeLkwberholverbotSicht</code>.
	 */
  private Data erzeugeAtgParameterLkwUeberholverbotSichtWochentag(int prueffall)
	{
		DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.sswSbaUeLkwberholverbotSicht"), m_dataModel.getAspect("asp.parameterSoll"));
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);

		// Aktuelle Parameter holen
		ResultData rdata = m_connection.getData(ue, dd, 1000);
		
		// Wert setzen
		ParameterLkwUeberholverbotSichtWochentag parameter = new ParameterLkwUeberholverbotSichtWochentag(rdata);
		if (prueffall == m_prueffall1)
		{
			parameter.getFahrstreifen2().setQBUevPEin(3200);		
			parameter.getFahrstreifen2().setQBUevPAus(2900);
			parameter.getFahrstreifen2().setQLkwPEin(650);
			parameter.getFahrstreifen2().setQLkwPAus(550);
			parameter.getFahrstreifen2().setSichtStufeEin(2);
			parameter.getFahrstreifen2().setSichtStufeAus(2);

			parameter.getFahrstreifen3().setQBUevPEin(4000);		
			parameter.getFahrstreifen3().setQBUevPEin(3600);
			parameter.getFahrstreifen3().setQLkwPEin(600);
			parameter.getFahrstreifen3().setQLkwPAus(500);
			parameter.getFahrstreifen3().setSichtStufeEin(2);
			parameter.getFahrstreifen3().setSichtStufeAus(2);

			parameter.getFahrstreifen4().setQBUevPEin(4400);		
			parameter.getFahrstreifen4().setQBUevPAus(3900);
			parameter.getFahrstreifen4().setQLkwPEin(650);
			parameter.getFahrstreifen4().setQLkwPAus(500);
			parameter.getFahrstreifen4().setSichtStufeEin(2);
			parameter.getFahrstreifen4().setSichtStufeAus(2);
			
			parameter.setEreignisTyp(m_ereignisTyp);
		}
		else if (prueffall == m_prueffall2)
		{
			parameter.getFahrstreifen2().setQBUevPEin(3200);		
			parameter.getFahrstreifen2().setQBUevPAus(2900);
			parameter.getFahrstreifen2().setQLkwPEin(650);
			parameter.getFahrstreifen2().setQLkwPAus(550);
			parameter.getFahrstreifen2().setSichtStufeEin(3);
			parameter.getFahrstreifen2().setSichtStufeAus(3);

			parameter.getFahrstreifen3().setQBUevPEin(4000);		
			parameter.getFahrstreifen3().setQBUevPEin(3600);
			parameter.getFahrstreifen3().setQLkwPEin(600);
			parameter.getFahrstreifen3().setQLkwPAus(500);
			parameter.getFahrstreifen3().setSichtStufeEin(2);
			parameter.getFahrstreifen3().setSichtStufeAus(2);

			parameter.getFahrstreifen4().setQBUevPEin(4400);		
			parameter.getFahrstreifen4().setQBUevPAus(3900);
			parameter.getFahrstreifen4().setQLkwPEin(650);
			parameter.getFahrstreifen4().setQLkwPAus(500);
			parameter.getFahrstreifen4().setSichtStufeEin(2);
			parameter.getFahrstreifen4().setSichtStufeAus(2);

			parameter.setEreignisTyp(m_ereignisTyp);
		}
		
		return parameter.erzeugeDatensatz((ClientDavConnection)m_connection);
	}

  /**
   * Methode erzeugt eine Manahmenanforderung mit der fr eine Anzeige ein bestimmter WVZ-Inhalt fr einen bestimmten
   * Schaltgrund angefordert wird
   * @param anzeige Anzeige die geschaltet werden soll
   * @return erzeugte Manahmenanforderung
   */
  private Massnahmenanforderung erzeugeMassnahmenAnforderung(SystemObject anzeige)
  {
    Massnahmenanforderung ma = new Massnahmenanforderung ();
    Vector<EinzelMassnahmeAlternative> einzelMassnahmeAlternative = new Vector<EinzelMassnahmeAlternative>();
    ma.setEinzelMassnahmeAlternative(einzelMassnahmeAlternative);
    
    EinzelMassnahmeAlternative alternative = new EinzelMassnahmeAlternative();
    einzelMassnahmeAlternative.add(alternative);
    
    Vector<AnzeigenDatensatz> datenSaetze = new Vector<AnzeigenDatensatz>();
    alternative.setEinzelMassnahmeAlternative(datenSaetze);
        
    AnzeigenDatensatz datensatz = new AnzeigenDatensatz();
    datenSaetze.add(datensatz);
    
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaMassnahmenAnforderung"));
		dataNeu.getArray("EinzelMassnahmeAlternative").setLength(1);
		dataNeu.getArray("EinzelMassnahmeAlternative").getItem(0).getArray("AnzeigenDatenstze").setLength(1);
		Data dataWvzInhalt = dataNeu.getArray("EinzelMassnahmeAlternative").getItem(0).getArray("AnzeigenDatenstze").getItem(0).getItem("WVZInhalt"); 

		SystemObject anzeigeInhalt = m_dataModel.getObject("bw.uz.B27.streckenBeeinflussungsAnlage.wvzInhalt.tlsGefahrenstelle");
		
		// WVZInhalt
		dataWvzInhalt.getReferenceValue("AnzeigeInhalt").setSystemObject(anzeigeInhalt);
		dataWvzInhalt.getScaledValue("Blinken").setText("Nein");
		dataWvzInhalt.getScaledValue("Blinktakt").set(-1);
		dataWvzInhalt.getArray("GrafikDarstellungen").setLength(0);
		dataWvzInhalt.getReferenceValue("Wechseltext").setText("undefiniert");
    
    datensatz.setAnzeige(anzeige);
    datensatz.setAnzeigeInhalt(anzeigeInhalt);
    datensatz.setAbsolutNotwendig(false);
    datensatz.setNotwendig(false);
    datensatz.setPrioritt(-1);
    datensatz.setVerschiebbarNachfolger(0);
    datensatz.setVerschiebbarVorgnger(0);
    datensatz.setWvzInhalt(dataWvzInhalt);
    
    return ma;
  }
	/**
	 * Diese Methode parametriert das bergebene Objekt mit den bergebenen Datenstze.
	 * Wird <code>null</code> bergeben wird die entsprechende Attributgruppe nicht
	 * parametriert.
	 * 
	 * @param ue Dyn. Objekt Ursacheneinheit.
	 * @param atgSp Datensatz <code>atg.sswSbaUeLkwberholverbotSicht</code>.
	 * @param atgEm Datensatz <code>atg.sswSbaEinzelMassnahme</code>.
	 * @param atgUe Datensatz <code>atg.sswSbaUrsachenEinheit</code>.
	 * @param atgMq Datensatz <code>atg.sswSbaUeMqZuordnung</code>.
	 * @param atgUfds Datensatz <code>atg.sswSbaUeUfdsZuordnung</code>.
	 */
	private void parametriereDynObjUrsachenEinheit(DynamicObject ue, Data atgParam, Data atgEm, Data atgUe, Data atgMq, Data atgUfds)
	{
		if (ue != null)
		{
			System.out.println("Ursacheneinheit " + ue + " parametrieren");
			
			Aspect asp = m_dataModel.getAspect("asp.parameterVorgabe");

      if (atgParam != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUeLkwberholverbotSicht");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgParam, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }
      
      if (atgEm != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaEinzelMassnahme");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgEm, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }
      
      if (atgUe != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUrsachenEinheit");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgUe, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }

      if (atgMq != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUeMqZuordnung");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgMq, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }

      if (atgUfds != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUeUfdsZuordnung");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgUfds, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }

		}			
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.INeueDatenListener#neueDatenSituationAnalyseMq(de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.SituationAnalyseMq)
	 */
	@Override
	public void neueDatenSituationAnalyseMq(SituationAnalyseMq analyse)
	{
		m_analyseMq = analyse;
		m_analyseMqErhalten = true;
		
		synchronized (m_objekt2)
		{
			m_objekt2.notify();					
		}
	}

	@Override
	public void neueDatenStaukriterium(StaukriteriumErgebnis staukriterium)
	{
		// TODO Auto-generated method stub
		
	}

}
