/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lkwueberholverbotsichtwochentag;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Diese Klasse speichert die Daten der Attributgruppe !atg.sswSbaUeLkwberholverbotSichtWochentag!.<p>
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.1 $ / $Date: 2015/08/26 13:35:37 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class ParameterLkwUeberholverbotSichtWochentag
{
	/**
	 * Pid der Attributgruppe.
	 */
	private String m_pidAtg = "atg.sswSbaUeLkwberholverbotSichtWochentag";
	/**
	 * Parameterversorgung fr 2 Fahrstreifen.
	 */
	public ParameterLkwUeberholverbotSichtWochentagFahrstreifen m_fahrstreifen2;

	/**
	 * Parameterversorgung fr 3 Fahrstreifen.
	 */
	public ParameterLkwUeberholverbotSichtWochentagFahrstreifen m_fahrstreifen3;

	/**
	 * Parameterversorgung fr 4 Fahrstreifen.
	 */
	public ParameterLkwUeberholverbotSichtWochentagFahrstreifen m_fahrstreifen4;
	/**
	 * Referenz auf einen EreinisTyp.
	 */
	private SystemObject m_ereignisTyp;

	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public ParameterLkwUeberholverbotSichtWochentag()
	{
	}
	/**
	 * Konstruktor mit bernahme des bergebenen Datensatzes in die interne Struktur.
	 *  
	 * @param rdata Datensatz der Parameter.
	 */
	public ParameterLkwUeberholverbotSichtWochentag(ResultData rdata)
	{
		setdaten(rdata);
	}
	
	/**
	 * bernahme des bergebenen Datensatzes in die interne Struktur.
	 * 
	 * @param rdata Datensatz der Parameter.
	 */
	public void setdaten(ResultData rdata)
	{
		if (rdata != null)
		{
			Data data = rdata.getData();
			if (data != null)
			{
				if (m_fahrstreifen2 == null)
					m_fahrstreifen2 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();
				
				m_fahrstreifen2.setQLkwPEin(data.getItem("2Fahrstreifen").getUnscaledValue("QLkwPEin").intValue());
				m_fahrstreifen2.setQLkwPAus(data.getItem("2Fahrstreifen").getUnscaledValue("QLkwPAus").intValue());
				m_fahrstreifen2.setQBUevPEin(data.getItem("2Fahrstreifen").getUnscaledValue("QBvPEin").intValue());
				m_fahrstreifen2.setQBUevPAus(data.getItem("2Fahrstreifen").getUnscaledValue("QBvPAus").intValue());
				m_fahrstreifen2.setSichtStufeEin(data.getItem("2Fahrstreifen").getUnscaledValue("SichtstufeEin").intValue());
				m_fahrstreifen2.setSichtStufeAus(data.getItem("2Fahrstreifen").getUnscaledValue("SichtstufeAus").intValue());

				if (m_fahrstreifen3 == null)
					m_fahrstreifen3 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();
				
				m_fahrstreifen3.setQLkwPEin(data.getItem("3Fahrstreifen").getUnscaledValue("QLkwPEin").intValue());
				m_fahrstreifen3.setQLkwPAus(data.getItem("3Fahrstreifen").getUnscaledValue("QLkwPAus").intValue());
				m_fahrstreifen3.setQBUevPEin(data.getItem("3Fahrstreifen").getUnscaledValue("QBvPEin").intValue());
				m_fahrstreifen3.setQBUevPAus(data.getItem("3Fahrstreifen").getUnscaledValue("QBvPAus").intValue());
				m_fahrstreifen3.setSichtStufeEin(data.getItem("3Fahrstreifen").getUnscaledValue("SichtstufeEin").intValue());
				m_fahrstreifen3.setSichtStufeAus(data.getItem("3Fahrstreifen").getUnscaledValue("SichtstufeAus").intValue());

				if (m_fahrstreifen4 == null)
					m_fahrstreifen4 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();
				
				m_fahrstreifen4.setQLkwPEin(data.getItem("4Fahrstreifen").getUnscaledValue("QLkwPEin").intValue());
				m_fahrstreifen4.setQLkwPAus(data.getItem("4Fahrstreifen").getUnscaledValue("QLkwPAus").intValue());
				m_fahrstreifen4.setQBUevPEin(data.getItem("4Fahrstreifen").getUnscaledValue("QBvPEin").intValue());
				m_fahrstreifen4.setQBUevPAus(data.getItem("4Fahrstreifen").getUnscaledValue("QBvPAus").intValue());
				m_fahrstreifen4.setSichtStufeEin(data.getItem("4Fahrstreifen").getUnscaledValue("SichtstufeEin").intValue());
				m_fahrstreifen4.setSichtStufeAus(data.getItem("4Fahrstreifen").getUnscaledValue("SichtstufeAus").intValue());

				m_ereignisTyp = data.getReferenceValue("keinLkwFreierTag").getSystemObject();
			}
			else
			{
				m_fahrstreifen2 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();
				m_fahrstreifen3 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();
				m_fahrstreifen4 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();				
				
				m_ereignisTyp = null;
			}
		}		
	}

	/**
	 * Erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeLkwberholverbotBelastung</code> mit
	 * den Daten der internen Struktur.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @return erzeugter Datensatz.
	 */
	public Data erzeugeDatensatz(ClientDavConnection connection)
	{
		DataModel dm = connection.getDataModel();		
		Data dataNeu = connection.createData(dm.getAttributeGroup(m_pidAtg));

		if (dataNeu != null)
		{
			dataNeu.getItem("2Fahrstreifen").getUnscaledValue("QLkwPEin").set(m_fahrstreifen2.getQLkwPEin());
			dataNeu.getItem("2Fahrstreifen").getUnscaledValue("QLkwPAus").set(m_fahrstreifen2.getQLkwPAus());
			dataNeu.getItem("2Fahrstreifen").getUnscaledValue("QBvPEin").set(m_fahrstreifen2.getQBUevPEin());
			dataNeu.getItem("2Fahrstreifen").getUnscaledValue("QBvPAus").set(m_fahrstreifen2.getQBUevPAus());
			dataNeu.getItem("2Fahrstreifen").getUnscaledValue("SichtstufeEin").set(m_fahrstreifen2.getSichtStufeEin());
			dataNeu.getItem("2Fahrstreifen").getUnscaledValue("SichtstufeAus").set(m_fahrstreifen2.getSichtStufeAus());

			dataNeu.getItem("3Fahrstreifen").getUnscaledValue("QLkwPEin").set(m_fahrstreifen3.getQLkwPEin());
			dataNeu.getItem("3Fahrstreifen").getUnscaledValue("QLkwPAus").set(m_fahrstreifen3.getQLkwPAus());
			dataNeu.getItem("3Fahrstreifen").getUnscaledValue("QBvPEin").set(m_fahrstreifen3.getQBUevPEin());
			dataNeu.getItem("3Fahrstreifen").getUnscaledValue("QBvPAus").set(m_fahrstreifen3.getQBUevPAus());
			dataNeu.getItem("3Fahrstreifen").getUnscaledValue("SichtstufeEin").set(m_fahrstreifen3.getSichtStufeEin());
			dataNeu.getItem("3Fahrstreifen").getUnscaledValue("SichtstufeAus").set(m_fahrstreifen3.getSichtStufeAus());

			dataNeu.getItem("4Fahrstreifen").getUnscaledValue("QLkwPEin").set(m_fahrstreifen4.getQLkwPEin());
			dataNeu.getItem("4Fahrstreifen").getUnscaledValue("QLkwPAus").set(m_fahrstreifen4.getQLkwPAus());
			dataNeu.getItem("4Fahrstreifen").getUnscaledValue("QBvPEin").set(m_fahrstreifen4.getQBUevPEin());
			dataNeu.getItem("4Fahrstreifen").getUnscaledValue("QBvPAus").set(m_fahrstreifen4.getQBUevPAus());
			dataNeu.getItem("4Fahrstreifen").getUnscaledValue("SichtstufeEin").set(m_fahrstreifen4.getSichtStufeEin());
			dataNeu.getItem("4Fahrstreifen").getUnscaledValue("SichtstufeAus").set(m_fahrstreifen4.getSichtStufeAus());

			if (m_ereignisTyp != null)
				dataNeu.getReferenceValue("keinLkwFreierTag").setSystemObject(m_ereignisTyp);
			else
				dataNeu.getReferenceValue("keinLkwFreierTag").setText("undefiniert");

			return dataNeu;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() 
	{
		StringBuffer string = new StringBuffer();
		
		string.append("\n 2 Fahrstreifen " + m_fahrstreifen2 +
				          "\n 3 Fahrstreifen " + m_fahrstreifen3 +
				          "\n 4 Fahrstreifen " + m_fahrstreifen4 +
				          "\n EreignisTyp    " + m_ereignisTyp);
		
		return string.toString();
  }

	/**
	 * Gibt fr die bergebene Anzahl Fahrstreifen den entsprechenden Datensatz zurck.
	 * 
	 * @param anzFs Anzahl der Fahrstreifen.
	 * @return Parameterversorgung der bergebenen Anzahl von Fahrstreifen.
	 */
	public ParameterLkwUeberholverbotSichtWochentagFahrstreifen getParameterFahrstreifen(int anzFs)
	{
		switch (anzFs)
		{
			case 0:
			case 1:
			{				
				return null;
			}
			case 2:
			{				
				return m_fahrstreifen2;
			}
			case 3:
			{				
				return m_fahrstreifen3;
			}
		}
		return m_fahrstreifen4;
	}

	/**
	 * bernimmt die bergebenen Grenzwerte in die interne Struktur der Parameter fr Fahrstreifen 2.
	 * 
	 * @param qbUevEin Grenzwert Einschaltkriterium QB Prognose. 
	 * @param qbUevAus Grenzwert Ausschaltkriterium QB Prognose.
	 * @param qLkwPEin Grenzwert Einschaltkriterium QLkw Prognose.
	 * @param qLkwPAus Grenzwert Ausschaltkriterium QLkw Prognose.
	 * @param sichtEin Nssesufe Ein
	 * @param sichtAus Nssesufe Aus
	 */
	public void setWerteFahrstreifen2(int qbUevEin, int qbUevAus, int qLkwPEin, int qLkwPAus, int sichtEin, int sichtAus)
	{
		if (m_fahrstreifen2 == null)
			m_fahrstreifen2 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();
		
		m_fahrstreifen2.setQBUevPEin(qbUevEin);
		m_fahrstreifen2.setQBUevPAus(qbUevAus);
		m_fahrstreifen2.setQLkwPEin(qLkwPEin);		
		m_fahrstreifen2.setQLkwPAus(qLkwPAus);
		m_fahrstreifen2.setSichtStufeEin(sichtEin);
		m_fahrstreifen2.setSichtStufeAus(sichtAus);
	}

	/**
	 * bernimmt die bergebenen Grenzwerte in die interne Struktur der Parameter fr Fahrstreifen 3.
	 * 
	 * @param qbUevEin Grenzwert Einschaltkriterium QB Prognose. 
	 * @param qbUevAus Grenzwert Ausschaltkriterium QB Prognose.
	 * @param qLkwPEin Grenzwert Einschaltkriterium QLkw Prognose.
	 * @param qLkwPAus Grenzwert Ausschaltkriterium QLkw Prognose.
	 * @param sichtEin Nssesufe Ein
	 * @param sichtAus Nssesufe Aus
	 */
	public void setWerteFahrstreifen3(int qbUevEin, int qbUevAus, int qLkwPEin, int qLkwPAus, int sichtEin, int sichtAus)
	{
		if (m_fahrstreifen3 == null)
			m_fahrstreifen3 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();
		
		m_fahrstreifen3.setQBUevPEin(qbUevEin);
		m_fahrstreifen3.setQBUevPAus(qbUevAus);
		m_fahrstreifen3.setQLkwPEin(qLkwPEin);		
		m_fahrstreifen3.setQLkwPAus(qLkwPAus);
		m_fahrstreifen3.setSichtStufeEin(sichtEin);
		m_fahrstreifen3.setSichtStufeAus(sichtAus);
	}

	/**
	 * bernimmt die bergebenen Grenzwerte in die interne Struktur der Parameter fr Fahrstreifen 4.
	 * 
	 * @param qbUevEin Grenzwert Einschaltkriterium QB Prognose. 
	 * @param qbUevAus Grenzwert Ausschaltkriterium QB Prognose.
	 * @param qLkwPEin Grenzwert Einschaltkriterium QLkw Prognose.
	 * @param qLkwPAus Grenzwert Ausschaltkriterium QLkw Prognose.
	 * @param sichtEin Nssesufe Ein
	 * @param sichtAus Nssesufe Aus
	 */
	public void setWerteFahrstreifen4(int qbUevEin, int qbUevAus, int qLkwPEin, int qLkwPAus, int sichtEin, int sichtAus)
	{
		if (m_fahrstreifen4 == null)
			m_fahrstreifen4 = new ParameterLkwUeberholverbotSichtWochentagFahrstreifen();
		
		m_fahrstreifen4.setQBUevPEin(qbUevEin);
		m_fahrstreifen4.setQBUevPAus(qbUevAus);
		m_fahrstreifen4.setQLkwPEin(qLkwPEin);		
		m_fahrstreifen4.setQLkwPAus(qLkwPAus);
		m_fahrstreifen4.setSichtStufeEin(sichtEin);
		m_fahrstreifen4.setSichtStufeAus(sichtAus);
	}

	/**
	 * Gibt die Parameterversorgung fr 2 Fahrstreifen zurck.
	 * 
	 * @return Parameterversorgung fr 2 Fahrstreifen.
	 */
	public ParameterLkwUeberholverbotSichtWochentagFahrstreifen getFahrstreifen2()
	{
		return m_fahrstreifen2;
	}

	/**
	 * Setzt die Parameterversorgung fr 2 Fahrstreifen.
	 * 
	 * @param fahrstreifen2 zu setzende Parameterversorgung fr 2 Fahrstreifen.
	 */
	public void setFahrstreifen2(ParameterLkwUeberholverbotSichtWochentagFahrstreifen fahrstreifen2)
	{
		m_fahrstreifen2 = fahrstreifen2;
	}

	/**
	 * Gibt die Parameterversorgung fr 3 Fahrstreifen zurck.
	 * 
	 * @return Parameterversorgung fr 3 Fahrstreifen.
	 */
	public ParameterLkwUeberholverbotSichtWochentagFahrstreifen getFahrstreifen3()
	{
		return m_fahrstreifen3;
	}

	/**
	 * Setzt die Parameterversorgung fr 3 Fahrstreifen.
	 * 
	 * @param fahrstreifen3 zu setzende Parameterversorgung fr 3 Fahrstreifen.
	 */
	public void setFahrstreifen3(ParameterLkwUeberholverbotSichtWochentagFahrstreifen fahrstreifen3)
	{
		m_fahrstreifen3 = fahrstreifen3;
	}

	/**
	 * Gibt die Parameterversorgung fr 4 Fahrstreifen zurck.
	 * 
	 * @return Parameterversorgung fr 4 Fahrstreifen.
	 */
	public ParameterLkwUeberholverbotSichtWochentagFahrstreifen getFahrstreifen4()
	{
		return m_fahrstreifen4;
	}

	/**
	 * Setzt die Parameterversorgung fr 4 Fahrstreifen.
	 * 
	 * @param fahrstreifen4 zu setzende Parameterversorgung fr 4 Fahrstreifen.
	 */
	public void setFahrstreifen4(ParameterLkwUeberholverbotSichtWochentagFahrstreifen fahrstreifen4)
	{
		m_fahrstreifen4 = fahrstreifen4;
	}
	
  /**
	 * Gibt den Wert der Variable m_ereignisTyp zurck.
	 *
	 * @return Wert der Variable m_ereignisTyp.
	 */
	public SystemObject getEreignisTyp()
	{
		return m_ereignisTyp;
	}
	/**
	 * Setzt den Wert der Variable m_ereignisTyp.
	 *
	 * @param ereignisTyp zu setzender Wert der Variable m_ereignisTyp.
	 */
	public void setEreignisTyp(SystemObject ereignisTyp)
	{
		m_ereignisTyp = ereignisTyp;
	}

	/**
	 * Innere Klasse zum Speichern der Parameter pro Fahrstreifen.<p>
	 * ber Getter und Setter kann auf die Daten zugegriffen werden.
	 * 
   * @version $Revision: 1.1 $ / $Date: 2015/08/26 13:35:37 $ / ($Author: Asal $)
	 * @author  Swarco Traffic Systems GmbH
	 * @author  Matthias Obert
	 */
  public class ParameterLkwUeberholverbotSichtWochentagFahrstreifen
  {
  	/**
  	 * Grenzwert Einschaltkriterium QB Prognose in PkwE/h.
   	 */
  	protected int  m_qbUevPEin;
  	/**
  	 * Grenzwert Ausschaltkriterium QB Prognose in PkwE/h.
   	 */
  	protected int  m_qbUevPAus;
  	/**
  	 * Grenzwert Einschaltkriterium QLkw Prognose in Fz/h.
  	 */
  	protected int  m_qlkwPEin;
  	/**
  	 * Grenzwert Ausschaltkriterium QLkw Prognose in Fz/h.
  	 */
  	protected int  m_qlkwPAus;
  	/**
  	 * Nssestufe Ein.
  	 */
  	protected int m_sichtStufeEin;
  	/**
  	 * Nssestufe Aus.
  	 */
  	protected int m_sichtStufeAus;
  	
		/**
		 * Default-Konstruktor ohne weitere Funktion.
		 */
  	public ParameterLkwUeberholverbotSichtWochentagFahrstreifen()
		{
		}

  	/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() 
		{
			StringBuffer string = new StringBuffer();
			
			string.append(" QBvPEin " + m_qbUevPEin +
					          " QBvPAus " + m_qbUevPAus +
					          " QLkwPEin " + m_qlkwPEin +
					          " QLkqPAus " + m_qlkwPAus +
					          " SichtEin " + m_sichtStufeEin +
					          " SichtAus " + m_sichtStufeAus);
			
			return string.toString();
	  }

		/**
		 * Gibt den Grenzwert Einschaltkriterium QB Prognose in PkwE/h zurck.
		 * 
		 * @return Grenzwert Einschaltkriterium QB Prognose in PkwE/h.
		 */
		public int getQBUevPEin()
		{
			return m_qbUevPEin;
		}

		/**
		 * Setz den Grenzwert Einschaltkriterium QB Prognose in PkwE/h.
		 * 
		 * @param qbUevsEin zu setzender Grenzwert Einschaltkriterium QB Prognose in PkwE/h.
		 */
		public void setQBUevPEin(int qbUevsEin)
		{
			m_qbUevPEin = qbUevsEin;
		}

		/**
		 * Gibt den Grenzwert Ausschaltkriterium QB Prognose in PkwE/h zurck.
		 * 
		 * @return Grenzwert Ausschaltkriterium QB Prognose in PkwE/h.
		 */
		public int getQBUevPAus()
		{
			return m_qbUevPAus;
		}

		/**
		 * Setzt den Grenzwert Ausschaltkriterium QB Prognose in PkwE/h.
		 * 
		 * @param qbUevsAus zu setzender Grenzwert Ausschaltkriterium QB Prognose in PkwE/h.
		 */
		public void setQBUevPAus(int qbUevsAus)
		{
			m_qbUevPAus = qbUevsAus;
		}

		/**
		 * Gibt den Grenzwert Einschaltkriterium QLkw Prognose in Fz/h zurck.
		 * 
		 * @return Grenzwert Einschaltkriterium QLkw Prognose in Fz/h.
		 */
		public int getQLkwPEin()
		{
			return m_qlkwPEin;
		}

		/**
		 * Setzt den Grenzwert Einschaltkriterium QLkw Prognose in Fz/h.
		 * 
		 * @param qlkwUevEin zu setzender Grenzwert Einschaltkriterium QLkw Prognose in Fz/h.
		 */
		public void setQLkwPEin(int qlkwUevEin)
		{
			m_qlkwPEin = qlkwUevEin;
		}

		/**
		 * Gibt den Grenzwert Ausschaltkriterium QLkw Prognose in Fz/h zurck.
		 * 
		 * @return Grenzwert Ausschaltkriterium QLkw Prognose in Fz/h.
		 */
		public int getQLkwPAus()
		{
			return m_qlkwPAus;
		}

		/**
		 * Setzt den Grenzwert Ausschaltkriterium QLkw Prognose in Fz/h.
		 * 
		 * @param qlkwUevAus zu setzender Grenzwert Ausschaltkriterium QLkw Prognose in Fz/h.
		 */
		public void setQLkwPAus(int qlkwUevAus)
		{
			m_qlkwPAus = qlkwUevAus;
		}

		/**
		 * Gibt die Sichtstufe Ein zurck.
		 * 
		 * @return Sichtstufe Ein.
		 */
		public int getSichtStufeEin()
		{
			return m_sichtStufeEin;
		}

		/**
		 * Setzt die Sichtstufe Ein.
		 * 
		 * @param sichtStufeEin zu setzende Sichtstufe Ein.
		 */
		public void setSichtStufeEin(int sichtStufeEin)
		{
			m_sichtStufeEin = sichtStufeEin;
		}

		/**
		 * Gibt die Sichtstufe Aus zurck.
		 * 
		 * @return Sichtstufe Aus.
		 */
		public int getSichtStufeAus()
		{
			return m_sichtStufeAus;
		}

		/**
		 * Setzt die Sichtstufe Aus.
		 * 
		 * @param sichtStufeAus zu setzende Sichtstufe Aus.
		 */
		public void setSichtStufeAus(int sichtStufeAus)
		{
			m_sichtStufeAus = sichtStufeAus;
		}
  }
}